@@ -14,27 +14,16 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef HAVE_STDIO_H
-#include <stdio.h>
-#endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_LIBINTL_H
-#include <libintl.h>
-#endif
-
 #include <mousepad/mousepad-private.h>
 #include <mousepad/mousepad-application.h>
-
 #ifdef HAVE_DBUS
 #include <mousepad/mousepad-dbus.h>
 #endif
 
+#ifdef HAVE_LIBINTL_H
+#include <libintl.h>
+#endif
+
 
 /* globals */
 static gchar    **filenames = NULL;
@@ -72,6 +61,7 @@ main (gint argc, gchar **argv)
 
   /* bind the text domain to the locale directory */
   bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
   /* set the package textdomain */
   textdomain (GETTEXT_PACKAGE);
