@@ -124,7 +124,7 @@ mousepad_action_group_class_init (MousepadActionGroupClass *klass)
                 G_SIGNAL_RUN_LAST,
                 0, NULL, NULL,
                 g_cclosure_marshal_VOID__OBJECT,
-                G_TYPE_NONE, 0,
+                G_TYPE_NONE, 1,
                 GTK_SOURCE_TYPE_LANGUAGE);
 }
 
@@ -283,8 +283,7 @@ void
 mousepad_action_group_set_active_style_scheme (MousepadActionGroup  *self,
                                                GtkSourceStyleScheme *scheme)
 {
-  GtkAction *action;
-
+  GtkAction   *action;
   g_return_if_fail (MOUSEPAD_IS_ACTION_GROUP (self));
 
   if (GTK_SOURCE_IS_STYLE_SCHEME (self->active_scheme))
@@ -302,11 +301,6 @@ mousepad_action_group_set_active_style_scheme (MousepadActionGroup  *self,
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
   self->locked = FALSE;
 
-  /* update the setting when the active action is changed */
-  self->locked = TRUE;
-  MOUSEPAD_SETTING_SET_STRING (COLOR_SCHEME, gtk_source_style_scheme_get_id (scheme));
-  self->locked = FALSE;
-
   g_object_notify (G_OBJECT (self), "active-style-scheme");
 }
 
@@ -473,6 +467,8 @@ static void
 mousepad_action_group_style_scheme_action_activate (MousepadActionGroup       *self,
                                                     MousepadStyleSchemeAction *action)
 {
+  const gchar *scheme_name = NULL;
+
   /* only update the active action if we're not already in the process of
    * setting it and the sender action is actually active */
   if (! self->locked &&
@@ -481,7 +477,14 @@ mousepad_action_group_style_scheme_action_activate (MousepadActionGroup       *s
       GtkSourceStyleScheme *scheme;
 
       scheme = mousepad_style_scheme_action_get_style_scheme (action);
-      mousepad_action_group_set_active_style_scheme (self, scheme);
+
+      /* update the setting when the active action is changed */
+      if (scheme != NULL)
+        scheme_name = gtk_source_style_scheme_get_id(scheme);
+
+      self->locked = TRUE;
+      MOUSEPAD_SETTING_SET_STRING (COLOR_SCHEME, scheme_name);
+      self->locked = FALSE;
     }
 }
 
@@ -617,6 +620,13 @@ mousepad_action_group_get_sorted_section_names (void)
       language = gtk_source_language_manager_get_language (manager, *languages);
       if (G_LIKELY (GTK_SOURCE_IS_LANGUAGE (language)))
         {
+          /* ignore hidden languages */
+          if(gtk_source_language_get_hidden(language))
+            {
+              languages++;
+              continue;
+            }
+
           /* ensure no duplicates in list */
           if (!g_slist_find_custom (list,
                                     gtk_source_language_get_section (language),
@@ -651,6 +661,13 @@ mousepad_action_group_get_sorted_languages_for_section (const gchar *section)
       language = gtk_source_language_manager_get_language (manager, *languages);
       if (G_LIKELY (GTK_SOURCE_IS_LANGUAGE (language)))
         {
+          /* ignore hidden languages */
+          if(gtk_source_language_get_hidden(language))
+            {
+              languages++;
+              continue;
+            }
+
           /* only get languages in the specified section */
           if (g_strcmp0 (gtk_source_language_get_section (language), section) == 0)
             list = g_slist_prepend (list, language);
