@@ -14,33 +14,50 @@
  * Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
-#include <gdk/gdkkeysyms.h>
-
 #include <mousepad/mousepad-private.h>
+#include <mousepad/mousepad-gtkcompat.h>
+#include <mousepad/mousepad-settings.h>
 #include <mousepad/mousepad-util.h>
 #include <mousepad/mousepad-view.h>
 
+#include <gtksourceview/gtksourcebuffer.h>
+#include <gtksourceview/gtksourceview.h>
+#include <gtksourceview/gtksourcestylescheme.h>
+#include <gtksourceview/gtksourcestyleschememanager.h>
+
+/* GTK+ 3 removed textview->im_context which is used for multi-selection and
+ * nobody has re-written multi-selection feature to not use this field
+ * so if building with GTK3 or with GSEAL_ENABLE disable the multi-select
+ * feature altogether :( */
+#if ! GTK_CHECK_VERSION(3, 0, 0) && ! defined(GSEAL_ENABLE)
+#define HAVE_MULTISELECT 1
+#endif
 
 
+
+#define MOUSEPAD_VIEW_DEFAULT_FONT "Monospace"
 #define mousepad_view_get_buffer(view) (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)))
 
 
 
 static void      mousepad_view_finalize                      (GObject            *object);
+static void      mousepad_view_set_property                  (GObject            *object,
+                                                              guint               prop_id,
+                                                              const GValue       *value,
+                                                              GParamSpec         *pspec);
+static void      mousepad_view_get_property                  (GObject            *object,
+                                                              guint               prop_id,
+                                                              GValue             *value,
+                                                              GParamSpec         *pspec);
+#ifdef HAVE_MULTISELECT
 static void      mousepad_view_style_set                     (GtkWidget          *widget,
                                                               GtkStyle           *previous_style);
 static gint      mousepad_view_expose                        (GtkWidget          *widget,
                                                               GdkEventExpose     *event);
+#endif
 static gboolean  mousepad_view_key_press_event               (GtkWidget          *widget,
                                                               GdkEventKey        *event);
+#ifdef HAVE_MULTISELECT
 static gboolean  mousepad_view_button_press_event            (GtkWidget          *widget,
                                                               GdkEventButton     *event);
 static gboolean  mousepad_view_button_release_event          (GtkWidget          *widget,
@@ -62,6 +79,7 @@ static void      mousepad_view_selection_draw                (MousepadView
 static gboolean  mousepad_view_selection_timeout             (gpointer            user_data);
 static void      mousepad_view_selection_timeout_destroy     (gpointer            user_data);
 static gchar    *mousepad_view_selection_string              (MousepadView       *view);
+#endif
 static void      mousepad_view_indent_increase               (MousepadView       *view,
                                                               GtkTextIter        *iter);
 static void      mousepad_view_indent_decrease               (MousepadView       *view,
@@ -69,8 +87,10 @@ static void      mousepad_view_indent_decrease               (MousepadView
 static void      mousepad_view_indent_selection              (MousepadView       *view,
                                                               gboolean            increase,
                                                               gboolean            force);
+#ifdef HAVE_MULTISELECT
 static void      mousepad_view_transpose_multi_selection     (GtkTextBuffer       *buffer,
                                                               MousepadView        *view);
+#endif
 static void      mousepad_view_transpose_range               (GtkTextBuffer       *buffer,
                                                               GtkTextIter         *start_iter,
                                                               GtkTextIter         *end_iter);
@@ -79,6 +99,7 @@ static void      mousepad_view_transpose_lines               (GtkTextBuffer
                                                               GtkTextIter         *end_iter);
 static void      mousepad_view_transpose_words               (GtkTextBuffer       *buffer,
                                                               GtkTextIter         *iter);
+static void      mousepad_view_update_font                   (MousepadView        *view);
 
 
 
@@ -89,33 +110,59 @@ struct _MousepadViewClass
 
 struct _MousepadView
 {
-  GtkSourceView  __parent__;
+  GtkSourceView         __parent__;
 
   /* the selection style tag */
-  GtkTextTag         *selection_tag;
+  GtkTextTag           *selection_tag;
 
   /* list with all the selection marks */
-  GSList             *selection_marks;
+  GSList               *selection_marks;
 
   /* selection timeout id */
-  guint               selection_timeout_id;
+  guint                 selection_timeout_id;
 
   /* coordinates for the selection */
-  gint                selection_start_x;
-  gint                selection_start_y;
-  gint                selection_end_x;
-  gint                selection_end_y;
+  gint                  selection_start_x;
+  gint                  selection_start_y;
+  gint                  selection_end_x;
+  gint                  selection_end_y;
 
   /* length of the selection (-1 = zerro width selection, 0 = no selection) */
-  gint                selection_length;
+  gint                  selection_length;
 
   /* if the selection is in editing mode */
-  guint               selection_editing : 1;
+  guint                 selection_editing : 1;
+
+  /* the font used in the view */
+  gchar                *font_name;
+  PangoFontDescription *font_desc;
+
+  /* whitespace visualization */
+  gboolean              show_whitespace;
+  gboolean              show_line_endings;
+
+  gchar                *color_scheme;
+
+  gboolean              match_braces;
 };
 
 
 
-G_DEFINE_TYPE (MousepadView, mousepad_view, GTK_TYPE_SOURCE_VIEW);
+enum
+{
+  PROP_0,
+  PROP_FONT_NAME,
+  PROP_SHOW_WHITESPACE,
+  PROP_SHOW_LINE_ENDINGS,
+  PROP_COLOR_SCHEME,
+  PROP_WORD_WRAP,
+  PROP_MATCH_BRACES,
+  NUM_PROPERTIES
+};
+
+
+
+G_DEFINE_TYPE (MousepadView, mousepad_view, GTK_SOURCE_TYPE_VIEW)
 
 
 
@@ -127,13 +174,105 @@ mousepad_view_class_init (MousepadViewClass *klass)
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = mousepad_view_finalize;
+  gobject_class->set_property = mousepad_view_set_property;
+  gobject_class->get_property = mousepad_view_get_property;
 
   widget_class = GTK_WIDGET_CLASS (klass);
+  widget_class->key_press_event      = mousepad_view_key_press_event;
+#ifdef HAVE_MULTISELECT
   widget_class->expose_event         = mousepad_view_expose;
   widget_class->style_set            = mousepad_view_style_set;
-  widget_class->key_press_event      = mousepad_view_key_press_event;
   widget_class->button_press_event   = mousepad_view_button_press_event;
   widget_class->button_release_event = mousepad_view_button_release_event;
+#endif
+
+  g_object_class_install_property (
+    gobject_class,
+    PROP_FONT_NAME,
+    g_param_spec_string ("font-name",
+                         "FontName",
+                         "The name of the font to use in the view",
+                         MOUSEPAD_VIEW_DEFAULT_FONT,
+                         G_PARAM_READWRITE));
+
+  g_object_class_install_property (
+    gobject_class,
+    PROP_SHOW_WHITESPACE,
+    g_param_spec_boolean ("show-whitespace",
+                          "ShowWhitespace",
+                          "Whether whitespace is visualized in the view",
+                          FALSE,
+                          G_PARAM_READWRITE));
+
+  g_object_class_install_property (
+    gobject_class,
+    PROP_SHOW_LINE_ENDINGS,
+    g_param_spec_boolean ("show-line-endings",
+                          "ShowLineEndings",
+                          "Whether line-endings are visualized in the view",
+                          FALSE,
+                          G_PARAM_READWRITE));
+
+  g_object_class_install_property (
+    gobject_class,
+    PROP_COLOR_SCHEME,
+    g_param_spec_string ("color-scheme",
+                         "ColorScheme",
+                         "The id of the syntax highlighting color scheme to use",
+                         NULL,
+                         G_PARAM_READWRITE));
+
+  g_object_class_install_property (
+    gobject_class,
+    PROP_WORD_WRAP,
+    g_param_spec_boolean ("word-wrap",
+                          "WordWrap",
+                          "Whether to virtually wrap long lines in the view",
+                          FALSE,
+                          G_PARAM_READWRITE));
+
+  g_object_class_install_property (
+    gobject_class,
+    PROP_MATCH_BRACES,
+    g_param_spec_boolean ("match-braces",
+                          "MatchBraces",
+                          "Whether to highlight matching braces, parens, brackets, etc.",
+                          FALSE,
+                          G_PARAM_READWRITE));
+}
+
+
+
+static void
+mousepad_view_buffer_changed (MousepadView *view,
+                              GParamSpec   *pspec,
+                              gpointer      user_data)
+{
+  GtkSourceBuffer *buffer;
+
+  buffer = (GtkSourceBuffer*) gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+  if (GTK_SOURCE_IS_BUFFER (buffer))
+    {
+      GtkSourceStyleSchemeManager *manager;
+      GtkSourceStyleScheme        *scheme;
+
+      manager = gtk_source_style_scheme_manager_get_default ();
+      scheme = gtk_source_style_scheme_manager_get_scheme (manager,
+        view->color_scheme ? view->color_scheme : "");
+      gtk_source_buffer_set_style_scheme (buffer, scheme);
+
+      gtk_source_buffer_set_highlight_matching_brackets (buffer, view->match_braces);
+    }
+}
+
+
+
+static void
+mousepad_view_use_default_font_setting_changed (MousepadView *view,
+                                                gchar        *key,
+                                                GSettings    *settings)
+{
+  mousepad_view_update_font (view);
 }
 
 
@@ -147,13 +286,55 @@ mousepad_view_init (MousepadView *view)
   view->selection_marks = NULL;
   view->selection_length = 0;
   view->selection_editing = FALSE;
+  view->color_scheme = g_strdup ("none");
+  view->font_name = NULL;
+  view->font_desc = NULL;
+  view->match_braces = FALSE;
+
+  /* make sure any buffers set on the view get the color scheme applied to them */
+  g_signal_connect (view,
+                    "notify::buffer",
+                    G_CALLBACK (mousepad_view_buffer_changed),
+                    NULL);
 
   /* reset drag coordinates */
   view->selection_start_x = view->selection_end_x = -1;
   view->selection_start_y = view->selection_end_y = -1;
 
+#ifdef HAVE_MULTISELECT
   g_signal_connect (GTK_TEXT_VIEW (view)->im_context, "commit",
                     G_CALLBACK (mousepad_view_commit_handler), view);
+#endif
+
+  /* bind Gsettings */
+#define BIND_(setting, prop) \
+  MOUSEPAD_SETTING_BIND (setting, view, prop, G_SETTINGS_BIND_DEFAULT)
+
+  BIND_ (AUTO_INDENT,            "auto-indent");
+  BIND_ (FONT_NAME,              "font-name");
+  BIND_ (SHOW_WHITESPACE,        "show-whitespace");
+  BIND_ (SHOW_LINE_ENDINGS,      "show-line-endings");
+  BIND_ (HIGHLIGHT_CURRENT_LINE, "highlight-current-line");
+  BIND_ (INDENT_ON_TAB,          "indent-on-tab");
+  BIND_ (INDENT_WIDTH,           "indent-width");
+  BIND_ (INSERT_SPACES,          "insert-spaces-instead-of-tabs");
+  BIND_ (RIGHT_MARGIN_POSITION,  "right-margin-position");
+  BIND_ (SHOW_LINE_MARKS,        "show-line-marks");
+  BIND_ (SHOW_LINE_NUMBERS,      "show-line-numbers");
+  BIND_ (SHOW_RIGHT_MARGIN,      "show-right-margin");
+  BIND_ (SMART_HOME_END,         "smart-home-end");
+  BIND_ (TAB_WIDTH,              "tab-width");
+  BIND_ (COLOR_SCHEME,           "color-scheme");
+  BIND_ (WORD_WRAP,              "word-wrap");
+  BIND_ (MATCH_BRACES,           "match-braces");
+
+  /* override with default font when the setting is enabled */
+  MOUSEPAD_SETTING_CONNECT_OBJECT (USE_DEFAULT_FONT,
+                                   G_CALLBACK (mousepad_view_use_default_font_setting_changed),
+                                   view,
+                                   G_CONNECT_SWAPPED);
+
+#undef BIND_
 }
 
 
@@ -171,11 +352,87 @@ mousepad_view_finalize (GObject *object)
   if (G_UNLIKELY (view->selection_marks != NULL))
     g_slist_free (view->selection_marks);
 
+  /* cleanup color scheme name */
+  g_free (view->color_scheme);
+
   (*G_OBJECT_CLASS (mousepad_view_parent_class)->finalize) (object);
 }
 
 
 
+static void
+mousepad_view_set_property (GObject      *object,
+                            guint         prop_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
+{
+  MousepadView *view = MOUSEPAD_VIEW (object);
+
+  switch (prop_id)
+    {
+    case PROP_FONT_NAME:
+      mousepad_view_set_font_name (view, g_value_get_string (value));
+      break;
+    case PROP_SHOW_WHITESPACE:
+      mousepad_view_set_show_whitespace (view, g_value_get_boolean (value));
+      break;
+    case PROP_SHOW_LINE_ENDINGS:
+      mousepad_view_set_show_line_endings (view, g_value_get_boolean (value));
+      break;
+    case PROP_COLOR_SCHEME:
+      mousepad_view_set_color_scheme (view, g_value_get_string (value));
+      break;
+    case PROP_WORD_WRAP:
+      mousepad_view_set_word_wrap (view, g_value_get_boolean (value));
+      break;
+    case PROP_MATCH_BRACES:
+      mousepad_view_set_match_braces (view, g_value_get_boolean (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
+
+static void
+mousepad_view_get_property (GObject    *object,
+                            guint       prop_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
+{
+  MousepadView *view = MOUSEPAD_VIEW (object);
+
+  switch (prop_id)
+    {
+    case PROP_FONT_NAME:
+      g_value_set_string (value, mousepad_view_get_font_name (view));
+      break;
+    case PROP_SHOW_WHITESPACE:
+      g_value_set_boolean (value, mousepad_view_get_show_whitespace (view));
+      break;
+    case PROP_SHOW_LINE_ENDINGS:
+      g_value_set_boolean (value, mousepad_view_get_show_line_endings (view));
+      break;
+    case PROP_COLOR_SCHEME:
+      g_value_set_string (value, mousepad_view_get_color_scheme (view));
+      break;
+    case PROP_WORD_WRAP:
+      g_value_set_boolean (value, mousepad_view_get_word_wrap (view));
+      break;
+    case PROP_MATCH_BRACES:
+      g_value_set_boolean (value, mousepad_view_get_match_braces (view));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
+
+#ifdef HAVE_MULTISELECT
 static gboolean
 mousepad_view_expose (GtkWidget      *widget,
                       GdkEventExpose *event)
@@ -236,6 +493,7 @@ mousepad_view_style_set (GtkWidget *widget,
         mousepad_view_selection_draw (view, FALSE);
     }
 }
+#endif
 
 
 
@@ -248,7 +506,6 @@ mousepad_view_key_press_event (GtkWidget   *widget,
   GtkTextIter    iter;
   GtkTextMark   *cursor;
   guint          modifiers;
-  gboolean       im_handled;
   gboolean       is_editable;
 
   /* get the modifiers state */
@@ -321,13 +578,15 @@ mousepad_view_key_press_event (GtkWidget   *widget,
             /* delete or destroy the selection */
             if (view->selection_length > 0)
               mousepad_view_delete_selection (view);
+#ifdef HAVE_MULTISELECT
             else
               mousepad_view_selection_destroy (view);
-
+#endif
             return TRUE;
           }
         break;
 
+#ifdef HAVE_MULTISELECT
       case GDK_BackSpace:
         if (view->selection_marks != NULL && is_editable)
           {
@@ -337,10 +596,16 @@ mousepad_view_key_press_event (GtkWidget   *widget,
             return TRUE;
           }
         break;
+#endif
 
       default:
+#ifndef HAVE_MULTISELECT
+        break;
+#else
         if (G_UNLIKELY (view->selection_marks != NULL && is_editable))
           {
+            gboolean im_handled;
+
             /* block textview updates (from gtk) */
             gtk_text_view_set_editable (GTK_TEXT_VIEW (view), FALSE);
 
@@ -354,30 +619,34 @@ mousepad_view_key_press_event (GtkWidget   *widget,
             if (im_handled)
               return TRUE;
           }
+#endif
     }
 
+#ifdef HAVE_MULTISELECT
   /* remove the selection when no valid key combination has been pressed */
   if (G_UNLIKELY (view->selection_marks != NULL && event->is_modifier == FALSE))
     mousepad_view_selection_destroy (view);
+#endif
 
   return (*GTK_WIDGET_CLASS (mousepad_view_parent_class)->key_press_event) (widget, event);
 }
 
 
 
+#ifdef HAVE_MULTISELECT
 static void
 mousepad_view_commit_handler (GtkIMContext *context,
                               const gchar  *str,
                               MousepadView *view)
 {
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (GTK_IS_IM_CONTEXT (context));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (GTK_IS_IM_CONTEXT (context));
 
   /* if there is a selection, insert this string there too */
   if (G_UNLIKELY (view->selection_marks != NULL))
     {
       /* debug check */
-      mousepad_return_if_fail (gtk_text_view_get_editable (GTK_TEXT_VIEW (view)) == FALSE);
+      g_return_if_fail (gtk_text_view_get_editable (GTK_TEXT_VIEW (view)) == FALSE);
 
       /* handle the text input for the multi selection */
       mousepad_view_selection_key_press_event (view, str, 0, 0);
@@ -465,12 +734,14 @@ mousepad_view_button_release_event (GtkWidget      *widget,
 
   return (*GTK_WIDGET_CLASS (mousepad_view_parent_class)->button_release_event) (widget, event);
 }
+#endif
 
 
 
 /**
  * Selection Functions
  **/
+#ifdef HAVE_MULTISELECT
 static gboolean
 mousepad_view_selection_word_range (const GtkTextIter *iter,
                                     GtkTextIter       *range_start,
@@ -521,11 +792,11 @@ mousepad_view_selection_key_press_event (MousepadView *view,
   GSList        *li;
   GtkTextBuffer *buffer;
 
-  mousepad_return_if_fail (view->selection_marks != NULL);
-  mousepad_return_if_fail (view->selection_start_x == -1);
-  mousepad_return_if_fail (view->selection_end_x == -1);
-  mousepad_return_if_fail ((keyval == 0 && text != NULL) || keyval != 0);
-  mousepad_return_if_fail (g_slist_length (view->selection_marks) % 2 == 0);
+  g_return_if_fail (view->selection_marks != NULL);
+  g_return_if_fail (view->selection_start_x == -1);
+  g_return_if_fail (view->selection_end_x == -1);
+  g_return_if_fail ((keyval == 0 && text != NULL) || keyval != 0);
+  g_return_if_fail (g_slist_length (view->selection_marks) % 2 == 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -586,9 +857,9 @@ mousepad_view_selection_delete_content (MousepadView *view)
   GtkTextIter    start_iter, end_iter;
   GSList        *li;
 
-  mousepad_return_if_fail (view->selection_marks != NULL);
-  mousepad_return_if_fail (view->selection_length > 0);
-  mousepad_return_if_fail (g_slist_length (view->selection_marks) % 2 == 0);
+  g_return_if_fail (view->selection_marks != NULL);
+  g_return_if_fail (view->selection_length > 0);
+  g_return_if_fail (g_slist_length (view->selection_marks) % 2 == 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -628,7 +899,7 @@ mousepad_view_selection_destroy (MousepadView *view)
   GSList        *li;
   GtkTextIter    start_iter, end_iter;
 
-  mousepad_return_if_fail (view->selection_marks != NULL);
+  g_return_if_fail (view->selection_marks != NULL);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -688,8 +959,8 @@ mousepad_view_selection_draw (MousepadView *view,
   GtkTextMark   *start_mark, *end_mark;
   GSList        *li;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (view->selection_marks == NULL || g_slist_length (view->selection_marks) % 2 == 0);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (view->selection_marks == NULL || g_slist_length (view->selection_marks) % 2 == 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -923,7 +1194,7 @@ mousepad_view_selection_string (MousepadView *view)
   GtkTextIter    start_iter, end_iter;
   GSList        *li;
 
-  mousepad_return_val_if_fail (view->selection_marks == NULL || g_slist_length (view->selection_marks) % 2 == 0, NULL);
+  g_return_val_if_fail (view->selection_marks == NULL || g_slist_length (view->selection_marks) % 2 == 0, NULL);
 
   /* create new string */
   string = g_string_new (NULL);
@@ -961,6 +1232,7 @@ mousepad_view_selection_string (MousepadView *view)
   /* return the string */
   return g_string_free (string, FALSE);
 }
+#endif
 
 
 
@@ -1113,7 +1385,7 @@ mousepad_view_scroll_to_cursor (MousepadView *view)
 {
   GtkTextBuffer *buffer;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1126,6 +1398,7 @@ mousepad_view_scroll_to_cursor (MousepadView *view)
 
 
 
+#ifdef HAVE_MULTISELECT
 static void
 mousepad_view_transpose_multi_selection (GtkTextBuffer *buffer,
                                          MousepadView  *view)
@@ -1134,7 +1407,7 @@ mousepad_view_transpose_multi_selection (GtkTextBuffer *buffer,
   GSList      *strings = NULL;
   GtkTextIter  start_iter, end_iter;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the strings and delete the existing strings */
   for (li = view->selection_marks; li != NULL; li = li->next)
@@ -1176,6 +1449,7 @@ mousepad_view_transpose_multi_selection (GtkTextBuffer *buffer,
   /* free list */
   g_slist_free (strings);
 }
+#endif
 
 
 
@@ -1362,7 +1636,7 @@ mousepad_view_transpose (MousepadView *view)
   GtkTextBuffer *buffer;
   GtkTextIter    sel_start, sel_end;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1370,12 +1644,15 @@ mousepad_view_transpose (MousepadView *view)
   /* begin user action */
   gtk_text_buffer_begin_user_action (buffer);
 
+#ifdef HAVE_MULTISELECT
   if (view->selection_marks != NULL)
     {
       /* transpose a multi selection */
       mousepad_view_transpose_multi_selection (buffer, view);
     }
-  else if (gtk_text_buffer_get_selection_bounds (buffer, &sel_start, &sel_end))
+  else
+#endif
+  if (gtk_text_buffer_get_selection_bounds (buffer, &sel_start, &sel_end))
     {
       /* if the selection is not on the same line, include the whole lines */
       if (gtk_text_iter_get_line (&sel_start) == gtk_text_iter_get_line (&sel_end))
@@ -1436,16 +1713,18 @@ mousepad_view_clipboard_cut (MousepadView *view)
 {
   GtkClipboard  *clipboard;
   GtkTextBuffer *buffer;
-  gchar         *string;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
 
   /* get the clipboard */
   clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view), GDK_SELECTION_CLIPBOARD);
 
+#ifdef HAVE_MULTISELECT
   if (view->selection_marks != NULL)
     {
+      gchar *string;
+
       /* get the selection string */
       string = mousepad_view_selection_string (view);
 
@@ -1462,6 +1741,7 @@ mousepad_view_clipboard_cut (MousepadView *view)
       mousepad_view_selection_destroy (view);
     }
   else
+#endif
     {
       /* get the buffer */
       buffer = mousepad_view_get_buffer (view);
@@ -1481,16 +1761,18 @@ mousepad_view_clipboard_copy (MousepadView *view)
 {
   GtkClipboard  *clipboard;
   GtkTextBuffer *buffer;
-  gchar         *string;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
 
   /* get the clipboard */
   clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view), GDK_SELECTION_CLIPBOARD);
 
+#ifdef HAVE_MULTISELECT
   if (view->selection_marks != NULL)
     {
+      gchar *string;
+
       /* get the selection string */
       string = mousepad_view_selection_string (view);
 
@@ -1501,6 +1783,7 @@ mousepad_view_clipboard_copy (MousepadView *view)
       g_free (string);
     }
   else
+#endif
     {
       /* get the buffer */
       buffer = mousepad_view_get_buffer (view);
@@ -1627,9 +1910,10 @@ mousepad_view_delete_selection (MousepadView *view)
 {
   GtkTextBuffer *buffer;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
 
+#ifdef HAVE_MULTISELECT
   if (view->selection_marks != NULL)
     {
       /* remove the text in our selection */
@@ -1639,6 +1923,7 @@ mousepad_view_delete_selection (MousepadView *view)
       mousepad_view_selection_destroy (view);
     }
   else
+#endif
     {
       /* get the buffer */
       buffer = mousepad_view_get_buffer (view);
@@ -1659,11 +1944,13 @@ mousepad_view_select_all (MousepadView *view)
   GtkTextIter    start, end;
   GtkTextBuffer *buffer;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
+#ifdef HAVE_MULTISELECT
   /* cleanup our selection */
   if (view->selection_marks != NULL)
     mousepad_view_selection_destroy (view);
+#endif
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1680,12 +1967,13 @@ mousepad_view_select_all (MousepadView *view)
 void
 mousepad_view_change_selection (MousepadView *view)
 {
+#ifdef HAVE_MULTISELECT
   GtkTextBuffer *buffer;
   GtkTextIter    start_iter, end_iter;
   GdkRectangle   rect;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (mousepad_view_get_selection_length (view, NULL) != 0);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) != 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1734,6 +2022,7 @@ mousepad_view_change_selection (MousepadView *view)
 
   /* allow notifications again */
   g_object_thaw_notify (G_OBJECT (buffer));
+#endif
 }
 
 
@@ -1749,8 +2038,8 @@ mousepad_view_convert_selection_case (MousepadView *view,
   gint           offset = -1;
   GSList        *li = NULL;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (mousepad_view_get_selection_length (view, NULL) > 0);
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -1804,7 +2093,7 @@ mousepad_view_convert_selection_case (MousepadView *view,
                 break;
 
               default:
-                mousepad_assert_not_reached ();
+                g_assert_not_reached ();
                 break;
             }
 
@@ -1812,7 +2101,7 @@ mousepad_view_convert_selection_case (MousepadView *view,
           if (G_LIKELY (converted && strcmp (text, converted) != 0))
             {
               /* debug check */
-              mousepad_return_if_fail (g_utf8_validate (converted, -1, NULL));
+              g_return_if_fail (g_utf8_validate (converted, -1, NULL));
 
               /* delete old string */
               gtk_text_buffer_delete (buffer, &start_iter, &end_iter);
@@ -1840,11 +2129,13 @@ mousepad_view_convert_selection_case (MousepadView *view,
       /* select range */
       gtk_text_buffer_select_range (buffer, &end_iter, &start_iter);
     }
+#ifdef HAVE_MULTISELECT
   else
     {
       /* redraw column selection */
       mousepad_view_selection_draw (view, FALSE);
     }
+#endif
 
   /* end user action */
   gtk_text_buffer_end_user_action (buffer);
@@ -1869,7 +2160,7 @@ mousepad_view_convert_spaces_and_tabs (MousepadView *view,
   gint           start_offset = -1;
   gchar         *string;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -2034,7 +2325,7 @@ mousepad_view_strip_trailing_spaces (MousepadView *view)
   gint           start, end, i;
   gunichar       c;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -2106,7 +2397,7 @@ mousepad_view_move_selection (MousepadView *view,
   gchar         *text;
   gboolean       insert_eol;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -2229,7 +2520,7 @@ mousepad_view_duplicate (MousepadView *view)
   gboolean       has_selection;
   gboolean       insert_eol = FALSE;
 
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* get the buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -2268,7 +2559,7 @@ void
 mousepad_view_indent (MousepadView *view,
                       gint          type)
 {
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
   /* run a forced indent of the line(s) */
   mousepad_view_indent_selection (view, type == INCREASE_INDENT, TRUE);
@@ -2276,51 +2567,6 @@ mousepad_view_indent (MousepadView *view,
 
 
 
-void
-mousepad_view_set_line_numbers (MousepadView *view,
-                                gboolean      line_numbers)
-{
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-
-  gtk_source_view_set_show_line_numbers (GTK_SOURCE_VIEW (view), line_numbers);
-}
-
-
-
-void
-mousepad_view_set_auto_indent (MousepadView *view,
-                               gboolean      auto_indent)
-{
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-
-  gtk_source_view_set_auto_indent (GTK_SOURCE_VIEW (view), auto_indent);
-}
-
-
-
-void
-mousepad_view_set_tab_size (MousepadView *view,
-                             gint          tab_size)
-{
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-  mousepad_return_if_fail (GTK_IS_TEXT_VIEW (view));
-
-  gtk_source_view_set_tab_width (GTK_SOURCE_VIEW (view), tab_size);
-}
-
-
-
-void
-mousepad_view_set_insert_spaces (MousepadView *view,
-                                 gboolean      insert_spaces)
-{
-  mousepad_return_if_fail (MOUSEPAD_IS_VIEW (view));
-
-  gtk_source_view_set_insert_spaces_instead_of_tabs (GTK_SOURCE_VIEW (view), insert_spaces);
-}
-
-
-
 gboolean
 mousepad_view_get_selection_length (MousepadView *view,
                                     gboolean     *is_column_selection)
@@ -2330,7 +2576,7 @@ mousepad_view_get_selection_length (MousepadView *view,
   gint           sel_length = 0;
   gboolean       column_selection;
 
-  mousepad_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
 
   /* get the text buffer */
   buffer = mousepad_view_get_buffer (view);
@@ -2360,40 +2606,229 @@ mousepad_view_get_selection_length (MousepadView *view,
 
 
 
-gboolean
-mousepad_view_get_line_numbers (MousepadView *view)
+void
+mousepad_view_set_font_name (MousepadView *view,
+                             const gchar  *font_name)
 {
-  mousepad_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+  PangoFontDescription *font_desc;
 
-  return gtk_source_view_get_show_line_numbers (GTK_SOURCE_VIEW (view));
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+
+  /* Passing NULL resets to the default font */
+  if (font_name == NULL)
+    font_name = MOUSEPAD_VIEW_DEFAULT_FONT;
+
+  font_desc = pango_font_description_from_string (font_name);
+  if (G_LIKELY (font_desc != NULL))
+    {
+      /* Save the normalized representation of the font as a string */
+      g_free (view->font_name);
+      view->font_name = pango_font_description_to_string (font_desc);
+
+      /* save the font description for later updating */
+      pango_font_description_free (view->font_desc);
+      view->font_desc = font_desc;
+
+      /* apply either the default or this font depending on settings */
+      mousepad_view_update_font (view);
+
+      /* Notify interested parties that the property has changed */
+      g_object_notify (G_OBJECT (view), "font-name");
+    }
+  else
+    g_critical ("Invalid font-name given: %s", font_name);
+}
+
+
+
+static void
+mousepad_view_update_draw_spaces (MousepadView *view)
+{
+  GtkSourceDrawSpacesFlags flags = 0;
+
+  if (view->show_whitespace)
+    {
+      flags |= GTK_SOURCE_DRAW_SPACES_SPACE |
+               GTK_SOURCE_DRAW_SPACES_TAB |
+               GTK_SOURCE_DRAW_SPACES_NBSP |
+               GTK_SOURCE_DRAW_SPACES_LEADING |
+               GTK_SOURCE_DRAW_SPACES_TEXT |
+               GTK_SOURCE_DRAW_SPACES_TRAILING;
+    }
+
+  if (view->show_line_endings)
+    flags |= GTK_SOURCE_DRAW_SPACES_NEWLINE;
+
+  gtk_source_view_set_draw_spaces (GTK_SOURCE_VIEW (view), flags);
+}
+
+
+
+static void
+mousepad_view_override_font (MousepadView         *view,
+                             PangoFontDescription *font_desc)
+{
+#if GTK_CHECK_VERSION (3, 0, 0)
+  gtk_widget_override_font (GTK_WIDGET (view), font_desc);
+#else
+  gtk_widget_modify_font (GTK_WIDGET (view), font_desc);
+#endif
+}
+
+
+
+static void
+mousepad_view_update_font (MousepadView *view)
+{
+  gboolean use_default;
+
+  use_default = MOUSEPAD_SETTING_GET_BOOLEAN (USE_DEFAULT_FONT);
+
+  if (use_default)
+    {
+      PangoFontDescription *font_desc;
+      font_desc = pango_font_description_from_string (MOUSEPAD_VIEW_DEFAULT_FONT);
+      mousepad_view_override_font (view, font_desc);
+      pango_font_description_free (font_desc);
+    }
+  else
+    mousepad_view_override_font (view, view->font_desc);
+}
+
+
+
+const gchar *
+mousepad_view_get_font_name (MousepadView *view)
+{
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), NULL);
+
+  return view->font_name;
+}
+
+
+
+void
+mousepad_view_set_show_whitespace (MousepadView *view,
+                                   gboolean      show)
+{
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+
+  view->show_whitespace = show;
+  mousepad_view_update_draw_spaces (view);
+  g_object_notify (G_OBJECT (view), "show-whitespace");
 }
 
 
 
 gboolean
-mousepad_view_get_auto_indent (MousepadView *view)
+mousepad_view_get_show_whitespace (MousepadView *view)
 {
-  mousepad_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
 
-  return gtk_source_view_get_auto_indent (GTK_SOURCE_VIEW (view));
+  return view->show_whitespace;
 }
 
 
 
-gint
-mousepad_view_get_tab_size (MousepadView *view)
+void
+mousepad_view_set_show_line_endings (MousepadView *view,
+                                     gboolean      show)
 {
-  mousepad_return_val_if_fail (MOUSEPAD_IS_VIEW (view), -1);
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
 
-  return gtk_source_view_get_tab_width (GTK_SOURCE_VIEW (view));
+  view->show_line_endings = show;
+  mousepad_view_update_draw_spaces (view);
+  g_object_notify (G_OBJECT (view), "show-line-endings");
 }
 
 
 
 gboolean
-mousepad_view_get_insert_spaces (MousepadView *view)
+mousepad_view_get_show_line_endings (MousepadView *view)
 {
-  mousepad_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
 
-  return gtk_source_view_get_insert_spaces_instead_of_tabs (GTK_SOURCE_VIEW (view));
+  return view->show_line_endings;
+}
+
+
+
+void
+mousepad_view_set_color_scheme (MousepadView *view,
+                                const gchar  *color_scheme)
+{
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+
+  if (g_strcmp0 (color_scheme, view->color_scheme) != 0)
+    {
+      g_free (view->color_scheme);
+      view->color_scheme = g_strdup (color_scheme);
+
+      /* update the buffer if there is one */
+      mousepad_view_buffer_changed (view, NULL, NULL);
+
+      g_object_notify (G_OBJECT (view), "color-scheme");
+    }
+}
+
+
+
+const gchar *
+mousepad_view_get_color_scheme (MousepadView *view)
+{
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), NULL);
+
+  return view->color_scheme;
+}
+
+
+
+void
+mousepad_view_set_word_wrap (MousepadView *view,
+                             gboolean      enabled)
+{
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+
+  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view),
+                               enabled ? GTK_WRAP_WORD : GTK_WRAP_NONE);
+  g_object_notify (G_OBJECT (view), "word-wrap");
+}
+
+
+
+gboolean
+mousepad_view_get_word_wrap (MousepadView *view)
+{
+  GtkWrapMode mode;
+
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+
+  mode = gtk_text_view_get_wrap_mode (GTK_TEXT_VIEW (view));
+
+  return (mode == GTK_WRAP_WORD);
+}
+
+
+
+void
+mousepad_view_set_match_braces (MousepadView *view,
+                                gboolean      enabled)
+{
+  g_return_if_fail (MOUSEPAD_IS_VIEW (view));
+
+  view->match_braces = enabled;
+
+  mousepad_view_buffer_changed (view, NULL, NULL);
+
+  g_object_notify (G_OBJECT (view), "match-braces");
+}
+
+
+
+gboolean
+mousepad_view_get_match_braces (MousepadView *view)
+{
+  g_return_val_if_fail (MOUSEPAD_IS_VIEW (view), FALSE);
+
+  return view->match_braces;
 }
